/******************************************************************************
 * (c) Copyright 2002-2005, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile: XMLToXSLTransformer.java,v $
 * Version:       $Name:  $ $Revision: 1.3 $
 * Last Modified: $Date: 2008/04/29 15:10:25 $
 *****************************************************************************/
package org.netkernel.xml.accessor.xslt;

import java.net.URI;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;
import org.netkernel.xml.util.NKFEntityResolver;
import org.netkernel.xml.util.NKFURIResolver;
import org.netkernel.xml.xda.IXDAReadOnly;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;


import javax.xml.transform.*;

import javax.xml.transform.Source;
import javax.xml.transform.SourceLocator;
import javax.xml.transform.TransformerException;
import javax.xml.transform.sax.SAXSource;

/**
 * XMLToXSLTransformer
 * @author  tab
 */
public class XMLToXSLTransformer extends StandardTransreptorImpl
{

	public XMLToXSLTransformer()
	{	this.declareThreadSafe();
		this.declareName("XSLCompiler");
		this.declareDescription("preparses or compiles XSL stylesheets");
		this.declareToRepresentation(RepXSLTransformer.class);
		this.declareToRepresentation(RepCompiledXSLTransformer.class);
	}
	
	public void onTransrept(INKFRequestContext aContext) throws Exception
	{	IReadableBinaryStreamRepresentation from=(IReadableBinaryStreamRepresentation)aContext.sourcePrimary(IReadableBinaryStreamRepresentation.class);
		//Set CWU to primary.
		aContext.setCWU(aContext.getThisRequest().getIdentifier());
		
		boolean compiled = aContext.getThisRequest().getRepresentationClass().equals(RepCompiledXSLTransformer.class);
		try
		{	RepXSLTransformer aspect;
			aContext.setCWU(aContext.getThisRequest().getIdentifier());
			NKFURIResolver resolver = new NKFURIResolver(aContext);
			if (compiled)
			{	aspect = new RepCompiledXSLTransformer(aContext,from, null,  resolver, aContext.getThisRequest().getIdentifier());
			}
			else
			{	aspect = new RepXSLTransformer(aContext,from, null, resolver, aContext.getThisRequest().getIdentifier());
			}
			resolver.cleanup();
			aContext.createResponseFrom(aspect);
			
		} catch (Exception e)
		{	//TO DO get this to work on JDK1.6
			SourceLocator sl = getRootSourceLocator(e);
			String location=null;
			if (sl!=null)
			{	location="line: "+sl.getLineNumber()+", column: "+sl.getColumnNumber();
			}
			NKFException nke = new NKFException("Failed to compile XSL",location,e);
			throw nke;
		}
	}
	
	
	public static SourceLocator getRootSourceLocator(Throwable exception)
	{
	  SourceLocator locator = null;
	  Throwable cause = exception;
	    
	  // Try to find the locator closest to the cause.
	  do
	  {
		  if (cause instanceof TransformerException)
		  {	locator = ((TransformerException)cause).getLocator();
		  }
		  cause = cause.getCause();
	  }
	  while (cause!=null && locator==null);
	        
	  return locator;
	}
}