package org.netkernel.xml.representation;


import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.xml.util.XMLUtils;

import org.w3c.dom.*;
import java.util.*;
import java.io.*;

/**
 * Serialise DOM to a binary stream
 * @author  tab
 */
public class RepSerializedHTML implements IBinaryStreamRepresentation
{
	Node mNode;
	boolean mIsHTML;
	
	private static Set sNonCoaleseElements = new HashSet();
	static 
	{	sNonCoaleseElements.add("textarea");
		sNonCoaleseElements.add("script");
	};
	
	public RepSerializedHTML(Node aNode, boolean aIsHTML)
	{	mNode = aNode;
		mIsHTML = aIsHTML;
	}
	
	public String getEncoding()
	{	return "UTF-8";
	}
	
	public void write(OutputStream aStream) throws IOException
	{	
		OutputStreamWriter osw = new OutputStreamWriter(aStream,"UTF-8");
		if (mIsHTML)
		{	//osw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			// this breaks mozilla- it won't load css links!
			osw.write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"  \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n");
			Element docEl = (Element)mNode;
			//docEl.setAttribute("xmlns", "http://www.w3.org/1999/xhtml");
			serialize(docEl,osw);
		}
		else
		{	try
			{	XMLUtils.toXML(osw, mNode, false,false,"UTF-8");
			}
			catch (Exception e)
			{	IOException ex = new IOException();
				ex.initCause(e);
				throw ex;
			}
		}
		osw.flush();
	}
	
	public static void serialize(Node aNode, Writer aWriter) throws IOException
	{	if (aNode!=null)
		{
			switch (aNode.getNodeType())
			{	case Node.DOCUMENT_NODE:
					serialize(((Document)aNode).getDocumentElement(),aWriter);
					break;
				case Node.ELEMENT_NODE:
				{	aWriter.write('<');
					String name = aNode.getNodeName();
					aWriter.write(name);
					NamedNodeMap atts = ((Element)aNode).getAttributes();
					int length=atts.getLength();
					if (length>0)
					{	aWriter.write(' ');
						for (int i=0; i<length; i++)
						{	serialize(atts.item(i),aWriter);
							aWriter.write(' ');
						}
					}
					Node n=aNode.getFirstChild();
					if (n==null && !sNonCoaleseElements.contains(name.toLowerCase()))
					{	aWriter.write(" />");
					}
					else
					{	aWriter.write(" >");
						while (n!=null)
						{	serialize(n,aWriter);
							n=n.getNextSibling();
						}
	
						aWriter.write("</");
						aWriter.write(aNode.getNodeName());
						aWriter.write('>');
					}
					
				}
				break;	
				case Node.ATTRIBUTE_NODE:
					aWriter.write(aNode.getNodeName());
					aWriter.write("=\"");
					writeAttr(aWriter, aNode.getNodeValue());
					//aWriter.write(aNode.getNodeValue());
					aWriter.write('"');
					break;
				case Node.TEXT_NODE:
				case Node.CDATA_SECTION_NODE:
					writeEscaped(aWriter, aNode.getNodeValue());
					break;
				case Node.COMMENT_NODE:
					aWriter.write("<!--");
					aWriter.write(aNode.getNodeValue());
					aWriter.write("-->");
					break;
			}
		}
	}
	
	public static void writeEscaped(Writer aWriter, String aInput) throws IOException
	{	char[] input = aInput.toCharArray();
		int length=input.length;
		for (int i=0; i<length; i++)
		{	char c=input[i];
			switch(c)
			{   case '<':
					aWriter.write("&#60;");
					break;
				case '>':
					aWriter.write("&#62;");
					break;
				case '\'':
					aWriter.write("&#39;");
					break;
				case '"':
					aWriter.write("&#34;");
					break;
				case '&':
					aWriter.write("&#38;");
					break;
				default:
					aWriter.write(c);
					break;
			}
		}
		
	}
	
	public static void writeAttr(Writer aWriter, String aInput) throws IOException
	{	char[] input = aInput.toCharArray();
		int length=input.length;
		for (int i=0; i<length; i++)
		{	char c=input[i];
			if (c=='"')
			{	aWriter.write("&#34;");
			}
			else
			{	aWriter.write(c);
			}
		}
	}
	
}