package org.netkernel.xml.xda;

import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;

import org.netkernel.layer0.util.PairList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

public class DOMNamespaceContext implements NamespaceContext
{
	private PairList mPrefix2NS = new PairList(4);
	
	public DOMNamespaceContext(Document aDocument)
	{
		if (aDocument!=null)
		{	Element e=aDocument.getDocumentElement();
			if(e!=null)
			{	String uri=e.getNamespaceURI();
				String prefix=e.getPrefix();
				//Take care of default namespace
				if(prefix==null)
				{	if(uri!=null)
					{	mPrefix2NS.put("",uri);
					}
					else
					{	mPrefix2NS.put("","");
					}
				}
			}
			
			buildPrefixResolver(aDocument);
		}
	}
	
	/**Find all xmlns namespace declarations and build a prefix resolver for xpath evaluation*/
    private void buildPrefixResolver(Node node)
    {	
		if(node.getNodeType()==Node.ELEMENT_NODE)
		{	NamedNodeMap nnm=node.getAttributes();
		    if(nnm!=null)
		    {	for(int i=0;i<nnm.getLength();i++)
				{	Node n=nnm.item(i);
					String name=n.getNodeName();
					if (name.startsWith("xmlns:"))
					{	String prefix=name.replaceAll("xmlns:", "");
						mPrefix2NS.put(prefix,n.getNodeValue());
					}
				}
		    }
	    }
		// recurse to all elements in document
		for (Node n=node.getFirstChild(); n!=null; n=n.getNextSibling())
		{   if (n instanceof Element)
		    {   buildPrefixResolver(n);
		    }
		}
	}
    
    public Iterator getPrefixes()
    {	ArrayList result = new ArrayList(4);
		int l=mPrefix2NS.size();
		for (int i=0; i<l; i++)
		{	result.add(mPrefix2NS.getValue1(i));
		}
		return result.iterator();
    }

	public String getNamespaceURI(String aPrefix)
	{	String result=XMLConstants.NULL_NS_URI;
		int l=mPrefix2NS.size();
		for (int i=0; i<l; i++)
		{	if (mPrefix2NS.getValue1(i).equals(aPrefix))
			{	result = (String)mPrefix2NS.getValue2(i);
				break;
			}
		}
		return result;
	}

	public String getPrefix(String aNamespaceURI)
	{	String result=XMLConstants.DEFAULT_NS_PREFIX;
		int l=mPrefix2NS.size();
		for (int i=0; i<l; i++)
		{	if (mPrefix2NS.getValue1(2).equals(aNamespaceURI))
			{	result = (String)mPrefix2NS.getValue1(i);
				break;
			}
		}
		return result;
	}

	public Iterator getPrefixes(String aNamespaceURI)
	{	ArrayList result = new ArrayList(4);
		int l=mPrefix2NS.size();
		for (int i=0; i<l; i++)
		{	if (mPrefix2NS.getValue1(2).equals(aNamespaceURI))
			{	result.add(mPrefix2NS.getValue1(i));
			}
		}
		return result.iterator();
	}

}
